<?php

namespace Drp\NativeEventApp\FileUploads;

use Aws\CommandInterface;
use Aws\S3\S3Client;
use Carbon\Carbon;
use Drp\NativeEventApp\FileUploads\Contracts\FileUploadDriver;
use Drp\NativeEventApp\Models\FileUpload;
use Illuminate\Contracts\Filesystem\FileNotFoundException;

class S3Driver implements FileUploadDriver
{
    public const CACHE_KEY_PREFIX = 'S3_IMAGES.';

    public function createUrl(string $uuid, string $expiry): array
    {
        $client = $this->storageClient();

        $signedRequest = $client->createPresignedRequest(
            $this->createPutCommand($client, $uuid),
            $expiry
        );

        return [
            'url' => (string) $signedRequest->getUri(),
            'headers' => json_encode(
                array_merge($signedRequest->getHeaders(), ['Content-Type' => 'application/octet-stream'])
            ),
        ];
    }

    public function storeReference(string $uuid, int $registrationId, ?string $type = null): FileUpload
    {
        return FileUpload::create([
            'uuid' => $uuid,
            'type' => $type ?? null,
            'disk' => config('native-event-app.files.s3.bucket'),
            'path' => $this->getFilePath($uuid),
            'registration_id' => $registrationId,
        ]);
    }

    public function getFileUrl(FileUpload $fileUpload): string
    {
        $expiry = Carbon::parse('+20 minute');

        return cache()->remember(
            $this->getCacheKey($fileUpload->uuid),
            $expiry,
            function () use ($fileUpload, $expiry) {
                $client = $this->storageClient();

                $signedRequest = $client->createPresignedRequest(
                    $this->createGetCommand($client, $fileUpload->uuid),
                    $expiry
                );

                return (string) $signedRequest->getUri();
            }
        );
    }

    public function getCacheKey(string $uuid): string
    {
        return self::CACHE_KEY_PREFIX . $uuid;
    }

    public function getStorageClient(): S3Client
    {
        return $this->storageClient();
    }

    protected function createPutCommand(S3Client $client, string $uuid): CommandInterface
    {
        return $client->getCommand('putObject', array_filter([
            'Bucket' => config('native-event-app.files.s3.bucket'),
            'Key' => $uuid,
            'ACL' => config('native-event-app.files.s3.visibility'),
            'ContentType' => 'application/octet-stream',
        ]));
    }

    protected function createGetCommand(S3Client $client, string $uuid): CommandInterface
    {
        return $client->getCommand('getObject', array_filter([
            'Bucket' => config('native-event-app.files.s3.bucket'),
            'Key' => $uuid,
        ]));
    }

    private function getFilePath(string $uuid): string
    {
        throw_unless(
            $this->storageClient()->doesObjectExist(config('native-event-app.files.s3.bucket'), $uuid),
            FileNotFoundException::class
        );

        return $uuid;
    }

    private function storageClient(): S3Client
    {
        return new S3Client([
            'region' => config('native-event-app.files.s3.region'),
            'version' => 'latest',
            'signature_version' => 'v4',
            'credentials' => array_filter([
                'key' => config('native-event-app.files.s3.key'),
                'secret' => config('native-event-app.files.s3.secret'),
                'token' => config('native-event-app.files.s3.token'),
                'url' => config('native-event-app.files.s3.url'),
            ]),
        ]);
    }
}
