<?php

namespace Drp\NativeEventApp\Test\Api;

use Drp\NativeEventApp\Models\Menu;
use Drp\NativeEventApp\Models\NavigatableItem;
use Drp\NativeEventApp\Models\Navigation;
use Drp\NativeEventApp\Models\Page;
use Drp\NativeEventApp\Models\Registration;
use Drp\NativeEventApp\Models\Segment;
use Spatie\Snapshots\MatchesSnapshots;

class MenuTest extends GraphQLTestCase
{
    use MatchesSnapshots;

    /** @test */
    public function can_get_menu(): void
    {
        [$user, $event] = $this->registerFakeEvent();

        Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);

        $page = factory(Page::class)->create();
        $primary = NavigatableItem::create([
            'icon' => 'house',
            'text' => 'Home',
            'destinationId' => $page->id,
            'destinationType' => Page::class,
        ]);
        $menu = Menu::create();
        $menu->items()->save($primary);

        Navigation::create([
            'primaryId' => $primary->id,
            'secondaryId' => $primary->id,
            'menuId' => $menu->id,
            'eventId' => $this->event->id,
        ]);

        $query = '{
            events(id:1){
                id,
                navigation {
                    primary {
                        icon
                        text
                    }
                    secondary {
                        icon
                        text
                    }
                    menu {
                        items {
                            icon
                            text
                            destination {
                                 __typename
                                ... on Page {
                                    id
                                }
                                ... on Link {
                                    href
                                }
                            }
                        }
                    }
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function orders_menu_items(): void
    {
        [$user, $event] = $this->registerFakeEvent();

        Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);

        $page = factory(Page::class)->create();
        $primary = NavigatableItem::create([
            'icon' => 'house',
            'text' => 'Home',
            'destinationId' => $page->id,
            'destinationType' => Page::class,
        ]);
        $secondary = NavigatableItem::create([
            'icon' => 'info',
            'text' => 'Information',
            'destinationId' => $page->id,
            'destinationType' => Page::class,
        ]);
        $menu = Menu::create();
        $menu->items()->attach($primary, ['order' => 2]);
        $menu->items()->attach($secondary, ['order' => 1]);

        Navigation::create([
            'primaryId' => $primary->id,
            'secondaryId' => $secondary->id,
            'menuId' => $menu->id,
            'eventId' => $this->event->id,
        ]);

        $query = '{
            events(id:1){
                id,
                navigation {
                    menu {
                        items {
                            icon
                            text
                            destination {
                                 __typename
                                ... on Page {
                                    id
                                }
                                ... on Link {
                                    href
                                }
                            }
                        }
                    }
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function can_scope_by_segments(): void
    {
        [$user, $event] = $this->registerFakeEvent();
        [$segmentA, $segmentB] = factory(Segment::class)->times(2)->create();

        $registration = Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);
        $registration->joinSegments(collect([$segmentA]));

        $page = factory(Page::class)->create();
        $primary = NavigatableItem::create([
            'icon' => 'house',
            'text' => 'Home',
            'destinationId' => $page->id,
            'destinationType' => Page::class,
        ]);
        $primary->joinSegments(collect([$segmentA]));

        $secondary = NavigatableItem::create([
            'icon' => 'hotel',
            'text' => 'Accomodation',
            'destinationId' => $page->id,
            'destinationType' => Page::class,
        ]);
        $secondary->joinSegments(collect([$segmentB]));

        $menu = Menu::create();
        $menu->items()->save($primary);
        $menu->items()->save($secondary);

        Navigation::create([
            'primaryId' => $primary->id,
            'secondaryId' => $secondary->id,
            'menuId' => $menu->id,
            'eventId' => $event->id,
        ]);

        $query = '{
            events(id:1){
                id,
                navigation {
                    primary {
                        icon
                        text
                    }
                    secondary {
                        icon
                        text
                    }
                    menu {
                        items {
                            icon
                            text
                            destination {
                                 __typename
                                ... on Page {
                                    id
                                }
                                ... on Link {
                                    href
                                }
                            }
                        }
                    }
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function gets_items_when_item_is_not_in_segment_but_registration_is()
    {
        [$user, $event] = $this->registerFakeEvent();
        $segmentA = factory(Segment::class)->create();

        // Create registration and assign segment
        $registration = Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);
        $registration->joinSegments(collect([$segmentA]));

        // Create navigation items, but dont assign segments
        $page = factory(Page::class)->create();
        $primary = NavigatableItem::create([
            'icon' => 'house',
            'text' => 'Home',
            'destinationId' => $page->id,
            'destinationType' => Page::class,
        ]);
        $secondary = NavigatableItem::create([
            'icon' => 'hotel',
            'text' => 'Accomodation',
            'destinationId' => $page->id,
            'destinationType' => Page::class,
        ]);

        $menu = Menu::create();
        $menu->items()->save($primary);
        $menu->items()->save($secondary);

        Navigation::create([
            'primaryId' => $primary->id,
            'secondaryId' => $secondary->id,
            'menuId' => $menu->id,
            'eventId' => $event->id,
        ]);

        $query = '{
            events(id:1){
                id,
                navigation {
                    primary {
                        icon
                        text
                    }
                    secondary {
                        icon
                        text
                    }
                    menu {
                        items {
                            icon
                            text
                            destination {
                                 __typename
                                ... on Page {
                                    id
                                }
                                ... on Link {
                                    href
                                }
                            }
                        }
                    }
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }

    /** @test */
    public function gets_all_navigation_items_if_registration_is_in_all_required_segments()
    {
        [$user, $event] = $this->registerFakeEvent();
        [$segmentA, $segmentB] = factory(Segment::class)->times(2)->create();

        $registration = Registration::create([
            'authId' => $user->getAuthIdentifier(),
            'eventId' => $event->id,
        ]);
        $registration->joinSegments(collect([$segmentA]));

        $page = factory(Page::class)->create();
        // Will return item, as registration is in all segments this item requires
        $primary = NavigatableItem::create([
            'icon' => 'house',
            'text' => 'Home',
            'destinationId' => $page->id,
            'destinationType' => Page::class,
        ]);
        $primary->joinSegments(collect([$segmentA]));

        // Will return null as registration only belongs to one segment this item requires
        $secondary = NavigatableItem::create([
            'icon' => 'hotel',
            'text' => 'Accomodation',
            'destinationId' => $page->id,
            'destinationType' => Page::class,
        ]);
        $secondary->joinSegments(collect([$segmentA, $segmentB]));

        $menu = Menu::create();
        $menu->items()->save($primary);
        $menu->items()->save($secondary);

        Navigation::create([
            'primaryId' => $primary->id,
            'secondaryId' => $secondary->id,
            'menuId' => $menu->id,
            'eventId' => $event->id,
        ]);

        $query = '{
            events(id:1){
                id,
                navigation {
                    primary {
                        icon
                        text
                    }
                    secondary {
                        icon
                        text
                    }
                    menu {
                        items {
                            icon
                            text
                            destination {
                                 __typename
                                ... on Page {
                                    id
                                }
                                ... on Link {
                                    href
                                }
                            }
                        }
                    }
                }
            }
        }';

        $response = $this->graphQlRequest($query);

        $this->assertMatchesJsonSnapshot($response);
    }
}
